#-----------------------------------------------------------------------------
# Name:        xmlConverter.py
# Purpose:     Various xml converter functions.
#
# Author:      Gerald Manipon
#
# Created:     2004/05/17
# Copyright:   (c) 2004, California Institute of Technology.
#              U.S. Government Sponsorship acknowledged.
# Licence:     
#-----------------------------------------------------------------------------
from xml.dom.minidom import getDOMImplementation, parseString

def simpleList2Xml(LL,rootElement = "Rows", rowElement = "row"):
    '''Create xml from a simple list'''
    
    #create root element
    implementation=getDOMImplementation()
    xmlDoc=implementation.createDocument(None,None,None)
    rootElem=xmlDoc.createElement(rootElement)
    xmlDoc.appendChild(rootElem)
    
    #loop over each record and populate dom
    for row in LL:
        
        #create row element
        rowElem=xmlDoc.createElement(rowElement)
        rowElem.appendChild(xmlDoc.createTextNode(str(row)))
        rootElem.appendChild(rowElem)
        
    #get string and return
    xmlString=xmlDoc.toxml()
    return xmlString

def xml2SimpleList(xmlString):
    '''Create a simple list from xml'''
    
    #create dom object
    domObj=parseString(xmlString)
    
    #get root element
    rootElem=domObj.documentElement
    rootElemName=rootElem.nodeName
    
    #get child nodes of root element
    childNodes=rootElem.childNodes
    
    #loop over childnodes and make sure they are
    #text_nodes.  If so, append to final list
    finalList=[]
    for node in childNodes:
        
        #make sure this is a text node only
        if node.firstChild.nodeType==3:
            finalList.append(node.firstChild.data)
            

    #return final list
    return finalList
    
def list2Xml(LL,headingsTuple = (), rootElement = "Rows", rowElement = "row"):
    '''Create xml from equal length lists of lists'''
    
    #get headings (will be tags within each row)
    if len(headingsTuple)==0:
        headingsTuple=LL.pop(0)
    
    #get number of headings
    numHeadings=len(headingsTuple)
    
    #get number of records
    numRecs=len(LL)
    
    #create root element
    implementation=getDOMImplementation()
    xmlDoc=implementation.createDocument(None,None,None)
    rootElem=xmlDoc.createElement(rootElement)
    xmlDoc.appendChild(rootElem)    
    
    #loop over each record, get heading and data, and populate dom
    for rec in LL:
        #check number of items in the rec, skip if not
        if len(rec)!=numHeadings:
            continue
        
        #create record element (corresponsds to a row)
        recElem=xmlDoc.createElement(rowElement)
        rootElem.appendChild(recElem)
        
        #loop over data and append
        for x in range(numHeadings):
            xElem=xmlDoc.createElement(headingsTuple[x])
            xElem.appendChild(xmlDoc.createTextNode(str(rec[x])))
            recElem.appendChild(xElem)
          
    #get string and return  
    xmlString=xmlDoc.toxml()
    return xmlString

def xml2List(xmlString):
    '''Create an equal length list of lists from xml'''
    
    #create dom object
    domObj=parseString(xmlString)
    
    #get root element
    rootElem=domObj.documentElement
    rootElemName=rootElem.nodeName
    
    #get child nodes of root element
    childNodes=rootElem.childNodes
    
    #number of child nodes (corresponds to number of lists in data structure)
    numChildNodes=childNodes.length
    
    #we'll get tagnames and number of items from the TEXT_NODES of
    #the first child
    firstChild=rootElem.firstChild
    tagNames=[]
    for node in firstChild.childNodes:
        
        #make sure this is a text node only
        if node.firstChild.nodeType==3:
            tagNames.append(node.nodeName)
            
    #print "Tagnames are:",',\n'.join(tagNames)
    
    #the list of tagnames is the first item in the final list of lists
    finalList=[]
    finalList.append(tagNames)

    #now loop over child nodes and get the value of the items within
    #each child node from the tagnames.
    #Append the list to the list of lists.
    #PLEASE NOTE: Since we extract the data according to the list of
    #text_node tagnames that we extracted from the first child, there is no
    #check for uniformity, except that the tagname exists, in the subsequent
    #childNodes.  Thus, additional tagnames (elements) can exist and should
    #not break this call.
    for i in range(numChildNodes):
        #get current iteration of child nodes
        node=childNodes.item(i)
        
        #get list of tagname nodes in current child node
        tagNodes=node.childNodes
        
        #get clean datalist
        datalist=[]
        
        #loop over tagnames
        for tagName in tagNames:
            
            #print "tagName is",tagName,"."
            
            #try to get the tagnode by tagName
            try:
                thisTagNode=node.getElementsByTagName(tagName)[0]
            except IndexError, e:
                raise RuntimeError, e
            
            #print "thisTagNode is",thisTagNode
            
            #get data value for the tag
            dataVal=thisTagNode.firstChild.data
            
            #append data value to datalist
            datalist.append(dataVal)
            
        #append datalist to finalList
        finalList.append(datalist)
        
    #return the final list
    return finalList
